package com.yonge.cooleshow.student.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.github.binarywang.wxpay.bean.notify.SignatureHeader;
import com.github.binarywang.wxpay.bean.notify.WxPayOrderNotifyV3Result;
import com.github.binarywang.wxpay.service.WxPayService;
import com.huifu.adapay.core.AdapayCore;
import com.huifu.adapay.core.util.AdapaySign;
import com.huifu.adapay.core.util.StringUtil;
import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.entity.UserOrder;
import com.yonge.cooleshow.biz.dal.entity.UserOrderPayment;
import com.yonge.cooleshow.biz.dal.enums.CacheNameEnum;
import com.yonge.cooleshow.biz.dal.service.UserOrderPaymentService;
import com.yonge.cooleshow.biz.dal.service.UserOrderRefundService;
import com.yonge.cooleshow.biz.dal.service.UserOrderService;
import com.yonge.cooleshow.biz.dal.support.DistributedLock;
import com.yonge.cooleshow.biz.dal.vo.UserOrderVo;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.toolset.base.exception.BizException;
import com.yonge.toolset.base.result.BaseResult;
import com.yonge.toolset.payment.base.enums.MethodNameEnum;
import com.yonge.toolset.payment.base.enums.OpenEnum;
import com.yonge.toolset.payment.base.enums.PayChannelEnum;
import com.yonge.toolset.payment.base.model.OpenAuth;
import com.yonge.toolset.payment.base.model.Payment;
import com.yonge.toolset.payment.base.model.callback.PaymentCallBack;
import com.yonge.toolset.payment.core.props.PaymentProperties;
import com.yonge.toolset.payment.core.service.PaymentClient;
import com.yonge.toolset.payment.core.service.SysConfigPaymentService;
import com.yonge.toolset.payment.original.wx.constant.WxpayConstant;
import com.yonge.toolset.utils.web.WebUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @Author: liweifan
 * @Data: 2022/3/11 18:16
 */
@RestController
@RequestMapping("/payment")
@Api(value = "付款接口", tags = "付款接口")
public class PaymentController extends BaseController {
    private final static Logger log = LoggerFactory.getLogger(PaymentController.class);
    @Autowired
    private UserOrderPaymentService paymentService;
    @Autowired
    private UserOrderRefundService userOrderRefundService;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private UserOrderService userOrderService;
    @Autowired
    private PaymentClient paymentClient;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private PaymentProperties paymentProperties;

    @ApiOperation(value = "查询付款单")
    @GetMapping("/queryPayment")
    public HttpResponseResult<Payment> queryPayment(String paymentNo) {
        UserOrderPayment userOrderPayment = paymentService.getOne(Wrappers.<UserOrderPayment>lambdaQuery()
                .eq(UserOrderPayment::getPaymentNo, paymentNo)
        );
        if (null == userOrderPayment) {
            return HttpResponseResult.succeed();
        }
        Payment param = new Payment(userOrderPayment.getOpenType(), userOrderPayment.getPayChannel());
        param.setId(userOrderPayment.getTransNo());
        param.setPaymentNo(userOrderPayment.getPaymentNo());
        BaseResult<Payment> paymentBaseResult = paymentClient.queryPayment(param);
        return HttpResponseResult.status(paymentBaseResult);
    }

    @ApiOperation(value = "通过用户给的授权码获取三方支付授权信息")
    @GetMapping("/getOpenId")
    public HttpResponseResult<Map<String, Object>> getOpenId(OpenAuth openAuth) {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        OpenEnum openType = OpenEnum.valueOf(paymentProperties.getOpenType());
        openAuth.setOpenType(openType);
        BaseResult<Map<String, Object>> res = paymentClient.getOpenAuthMsg(openAuth);
        return HttpResponseResult.status(res);
    }

    /***
     * 支付回调
     * @author liweifan
     * @param: request
     * @updateTime 2022/3/11 18:35
     */
    @PostMapping("/callback/{openType}/{payChannel}/{payMethod}")
    public Object callback(
            @PathVariable("openType") String openType,
            @PathVariable("payChannel") String payChannel,
            @PathVariable("payMethod") String payMethod,
            HttpServletRequest request
    ) {
        log.info("支付回调：openType is {} ，payChannel is {}，payMethod is {}", openType, payChannel, payMethod);
        BaseResult<PaymentCallBack> res = paymentClient.analysisNotice(
                OpenEnum.valueOf(openType), PayChannelEnum.valueOf(payChannel), MethodNameEnum.valueOf(payMethod), request);
        if (res.getStatus()) {
            PaymentCallBack data = res.getData();
            //查询到订单
            UserOrderVo userOrderVo = userOrderService.getUserOrderByPaymentNoOrTransNo(data.getPaymentNo(), data.getId());
            if(null == userOrderVo){
                return res.getData().getResMsg();
            }
            try {
                DistributedLock.of(redissonClient)
                        .runIfLockCanGet(CacheNameEnum.LOCK_EXECUTE_ORDER.getRedisKey(userOrderVo.getUserId())
                                , () -> {
                                    if (MethodNameEnum.executePayment.equals(data.getMethodName())) {
                                        //支付交易
                                        paymentService.executePaymentCallBack(data);
                                    } else if (MethodNameEnum.closePayment.equals(data.getMethodName())) {
                                        //关闭订单
                                        paymentService.closePaymentCallBack(data);
                                    } else if (MethodNameEnum.refundPayment.equals(data.getMethodName())) {
                                        //退款
                                        userOrderRefundService.refundPaymentCallBack(data);
                                    }
                                }, 60L, TimeUnit.SECONDS);
            } catch (BizException e) {
                log.error("回调业务异常，msg is {}", e.getMessage());
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return res.getData().getResMsg();
    }


    /***
     * 支付回调
     * @author liweifan
     * @param: request
     * @updateTime 2022/3/11 18:35
     */
    @PostMapping("/testCallback")
    public void testCallback(@RequestBody PaymentCallBack data) {
        if (MethodNameEnum.executePayment.equals(data.getMethodName())) {
            //支付交易
            paymentService.executePaymentCallBack(data);
        } else if (MethodNameEnum.closePayment.equals(data.getMethodName())) {
            //关闭订单
            paymentService.closePaymentCallBack(data);
        } else if (MethodNameEnum.refundPayment.equals(data.getMethodName())) {
            //退款
            userOrderRefundService.refundPaymentCallBack(data);
        }
    }
}
